package sun.awt.datatransfer;

import java.awt.datatransfer.SystemFlavorMap;
import java.awt.datatransfer.FlavorTable;
import java.util.Iterator;
import java.util.Arrays;
import java.awt.datatransfer.FlavorEvent;
import sun.awt.EventListenerAggregate;
import java.awt.datatransfer.FlavorListener;
import java.util.Set;
import sun.awt.SunToolkit;
import sun.awt.PeerEvent;
import java.beans.PropertyChangeEvent;
import java.io.IOException;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.datatransfer.DataFlavor;
import java.awt.EventQueue;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import sun.awt.AppContext;
import java.beans.PropertyChangeListener;
import java.awt.datatransfer.Clipboard;

public abstract class SunClipboard extends Clipboard implements PropertyChangeListener
{
    private AppContext contentsContext;
    private final Object CLIPBOARD_FLAVOR_LISTENER_KEY;
    private volatile int numberOfFlavorListeners;
    private volatile long[] currentFormats;
    
    public SunClipboard(String s) {
        super(s);
        this.contentsContext = null;
        this.numberOfFlavorListeners = 0;
        this.CLIPBOARD_FLAVOR_LISTENER_KEY = new StringBuffer(s + "_CLIPBOARD_FLAVOR_LISTENER_KEY");
    }
    
    @Override
    public synchronized void setContents(Transferable contentsNative, ClipboardOwner owner) {
        if (contentsNative == null) {
            throw new NullPointerException("contents");
        }
        this.initContext();
        final ClipboardOwner owner2 = this.owner;
        final Transferable contents = this.contents;
        try {
            this.owner = owner;
            this.contents = new TransferableProxy(contentsNative, true);
            this.setContentsNative(contentsNative);
        }
        finally {
            if (owner2 != null && owner2 != owner) {
                EventQueue.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        owner2.lostOwnership(SunClipboard.this, contents);
                    }
                });
            }
        }
        return;
    }
    
    private synchronized void initContext() {
        AppContext appContext = AppContext.getAppContext();
        if (this.contentsContext != appContext) {
            synchronized (appContext) {
                if (appContext.isDisposed()) {
                    throw new IllegalStateException("Can't set contents from disposed AppContext");
                }
                appContext.addPropertyChangeListener("disposed", this);
            }
            if (this.contentsContext != null) {
                this.contentsContext.removePropertyChangeListener("disposed", this);
            }
            this.contentsContext = appContext;
        }
    }
    
    @Override
    public synchronized Transferable getContents(Object o) {
        if (this.contents != null) {
            return this.contents;
        }
        return new ClipboardTransferable(this);
    }
    
    protected synchronized Transferable getContextContents() {
        return (AppContext.getAppContext() == this.contentsContext) ? this.contents : null;
    }
    
    @Override
    public DataFlavor[] getAvailableDataFlavors() {
        Transferable contextContents = this.getContextContents();
        if (contextContents != null) {
            return contextContents.getTransferDataFlavors();
        }
        return DataTransferer.getInstance().getFlavorsForFormatsAsArray(this.getClipboardFormatsOpenClose(), getDefaultFlavorTable());
    }
    
    @Override
    public boolean isDataFlavorAvailable(DataFlavor dataFlavor) {
        if (dataFlavor == null) {
            throw new NullPointerException("flavor");
        }
        Transferable contextContents = this.getContextContents();
        if (contextContents != null) {
            return contextContents.isDataFlavorSupported(dataFlavor);
        }
        return formatArrayAsDataFlavorSet(this.getClipboardFormatsOpenClose()).contains(dataFlavor);
    }
    
    @Override
    public Object getData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
        if (flavor == null) {
            throw new NullPointerException("flavor");
        }
        Transferable contextContents = this.getContextContents();
        if (contextContents != null) {
            return contextContents.getTransferData(flavor);
        }
        long longValue = 0L;
        byte[] clipboardData = null;
        Transferable localeTransferable = null;
        try {
            this.openClipboard(null);
            long[] clipboardFormats = this.getClipboardFormats();
            Long n = (Long)DataTransferer.getInstance().getFlavorsForFormats(clipboardFormats, getDefaultFlavorTable()).get(flavor);
            if (n == null) {
                throw new UnsupportedFlavorException(flavor);
            }
            longValue = (long)n;
            clipboardData = this.getClipboardData(longValue);
            if (DataTransferer.getInstance().isLocaleDependentTextFormat(longValue)) {
                localeTransferable = this.createLocaleTransferable(clipboardFormats);
            }
        }
        finally {
            this.closeClipboard();
        }
        return DataTransferer.getInstance().translateBytes(clipboardData, flavor, longValue, localeTransferable);
    }
    
    protected Transferable createLocaleTransferable(long[] array) throws IOException {
        return null;
    }
    
    public void openClipboard(SunClipboard sunClipboard) {
    }
    
    public void closeClipboard() {
    }
    
    public abstract long getID();
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if ("disposed".equals(propertyChangeEvent.getPropertyName()) && Boolean.TRUE.equals(propertyChangeEvent.getNewValue())) {
            this.lostOwnershipLater((AppContext)propertyChangeEvent.getSource());
        }
    }
    
    protected void lostOwnershipImpl() {
        this.lostOwnershipLater(null);
    }
    
    protected void lostOwnershipLater(AppContext appContext) {
        AppContext contentsContext = this.contentsContext;
        if (contentsContext == null) {
            return;
        }
        SunToolkit.postEvent(contentsContext, new PeerEvent(this, ()/*  */ -> this.lostOwnershipNow(p0), 1L));
        return;
    }
    
    protected void lostOwnershipNow(AppContext appContext) {
        ClipboardOwner owner = null;
        Transferable contents = null;
        synchronized (this) {
            AppContext contentsContext = this.contentsContext;
            if (contentsContext == null) {
                return;
            }
            if (appContext == null || contentsContext == appContext) {
                owner = this.owner;
                contents = this.contents;
                this.contentsContext = null;
                this.owner = null;
                this.contents = null;
                this.clearNativeContext();
                contentsContext.removePropertyChangeListener("disposed", this);
            }
            else {
                return;
            }
        }
        if (owner != null) {
            owner.lostOwnership(this, contents);
        }
    }
    
    protected abstract void clearNativeContext();
    
    protected abstract void setContentsNative(Transferable p0);
    
    protected long[] getClipboardFormatsOpenClose() {
        try {
            this.openClipboard(null);
            return this.getClipboardFormats();
        }
        finally {
            this.closeClipboard();
        }
    }
    
    protected abstract long[] getClipboardFormats();
    
    protected abstract byte[] getClipboardData(long p0) throws IOException;
    
    private static Set formatArrayAsDataFlavorSet(long[] array) {
        return (array == null) ? null : DataTransferer.getInstance().getFlavorsForFormatsAsSet(array, getDefaultFlavorTable());
    }
    
    @Override
    public synchronized void addFlavorListener(FlavorListener flavorListener) {
        if (flavorListener == null) {
            return;
        }
        AppContext appContext = AppContext.getAppContext();
        EventListenerAggregate value = (EventListenerAggregate)appContext.get(this.CLIPBOARD_FLAVOR_LISTENER_KEY);
        if (value == null) {
            value = new EventListenerAggregate(FlavorListener.class);
            appContext.put(this.CLIPBOARD_FLAVOR_LISTENER_KEY, value);
        }
        value.add(flavorListener);
        if (this.numberOfFlavorListeners++ == 0) {
            long[] clipboardFormats = null;
            try {
                this.openClipboard(null);
                clipboardFormats = this.getClipboardFormats();
            }
            catch (IllegalStateException ex) {}
            finally {
                this.closeClipboard();
            }
            this.currentFormats = clipboardFormats;
            this.registerClipboardViewerChecked();
        }
        return;
    }
    
    @Override
    public synchronized void removeFlavorListener(FlavorListener flavorListener) {
        if (flavorListener == null) {
            return;
        }
        EventListenerAggregate eventListenerAggregate = (EventListenerAggregate)AppContext.getAppContext().get(this.CLIPBOARD_FLAVOR_LISTENER_KEY);
        if (eventListenerAggregate == null) {
            return;
        }
        if (eventListenerAggregate.remove(flavorListener) && --this.numberOfFlavorListeners == 0) {
            this.unregisterClipboardViewerChecked();
            this.currentFormats = null;
        }
        return;
    }
    
    @Override
    public synchronized FlavorListener[] getFlavorListeners() {
        EventListenerAggregate eventListenerAggregate = (EventListenerAggregate)AppContext.getAppContext().get(this.CLIPBOARD_FLAVOR_LISTENER_KEY);
        return (eventListenerAggregate == null) ? new FlavorListener[0] : ((FlavorListener[])(FlavorListener[])eventListenerAggregate.getListenersCopy());
    }
    
    public boolean areFlavorListenersRegistered() {
        return this.numberOfFlavorListeners > 0;
    }
    
    protected abstract void registerClipboardViewerChecked();
    
    protected abstract void unregisterClipboardViewerChecked();
    
    protected final void checkChange(long[] array) {
        if (Arrays.equals(array, this.currentFormats)) {
            return;
        }
        this.currentFormats = array;
        for (AppContext appContext : AppContext.getAppContexts()) {
            if (appContext == null) {
                continue;
            }
            if (appContext.isDisposed()) {
                continue;
            }
            EventListenerAggregate eventListenerAggregate = (EventListenerAggregate)appContext.get(this.CLIPBOARD_FLAVOR_LISTENER_KEY);
            if (eventListenerAggregate != null) {
                FlavorListener[] array2 = (FlavorListener[])(FlavorListener[])eventListenerAggregate.getListenersInternal();
                for (int i = 0; i < array2.length; ++i) {
                    class SunFlavorChangeNotifier implements Runnable
                    {
                        private final FlavorListener flavorListener = array2[i];
                        
                        @Override
                        public void run() {
                            if (this.flavorListener != null) {
                                this.flavorListener.flavorsChanged(new FlavorEvent(SunClipboard.this));
                            }
                        }
                    }
                    SunToolkit.postEvent(appContext, new PeerEvent(this, new SunFlavorChangeNotifier(), 1L));
                }
            }
            else {
                continue;
            }
        }
        return;
    }
    
    public static FlavorTable getDefaultFlavorTable() {
        return (FlavorTable)SystemFlavorMap.getDefaultFlavorMap();
    }
}
